"use strict";
var ts = require('typescript');
var rollup_pluginutils_1 = require('rollup-pluginutils');
var fs_1 = require('fs');
var object_assign_1 = require('object-assign');
var compare_versions_1 = require('compare-versions');
var typescript_helpers_ts_1 = require('./typescript-helpers.ts');
var string_1 = require('./string');
var fixExportClass_1 = require('./fixExportClass');
var jsxOptions = {
    'preserve': ts.JsxEmit.Preserve,
    'react': ts.JsxEmit.React
};
var resolveHost = {
    fileExists: function (filePath) {
        try {
            return fs_1.statSync(filePath).isFile();
        }
        catch (err) {
            return false;
        }
    }
};
function goodErrors(diagnostic) {
    return diagnostic.code !== 1204;
}
function typescript(options) {
    options = object_assign_1.default({}, options || {});
    var filter = rollup_pluginutils_1.createFilter(options.include || ['*.ts+(|x)', '**/*.ts+(|x)'], options.exclude || ['*.d.ts', '**/*.d.ts']);
    delete options.include;
    delete options.exclude;
    var typescript = options.typescript || ts;
    if (typeof options.jsx === 'string') {
        options.jsx = jsxOptions[options.jsx] || ts.JsxEmit.None;
    }
    options = object_assign_1.default({
        noEmitHelpers: true,
        target: ts.ScriptTarget.ES5,
        module: ts.ModuleKind.ES6,
        sourceMap: true
    }, options);
    return {
        load: function (id) {
            if (id === 'typescript-helpers') {
                return typescript_helpers_ts_1.default;
            }
        },
        resolveId: function (importee, importer) {
            if (importee === 'typescript-helpers')
                return 'typescript-helpers';
            if (!importer)
                return null;
            var result;
            if (compare_versions_1.default(typescript.version, '1.8.0') < 0) {
                result = typescript.nodeModuleNameResolver(importee, importer, resolveHost);
            }
            else {
                result = typescript.nodeModuleNameResolver(importee, importer, options, resolveHost);
            }
            if (result.resolvedModule && result.resolvedModule.resolvedFileName) {
                if (string_1.endsWith(result.resolvedModule.resolvedFileName, '.d.ts')) {
                    return null;
                }
                return result.resolvedModule.resolvedFileName;
            }
            return null;
        },
        transform: function (code, id) {
            if (!filter(id))
                return null;
            var transformed = typescript.transpileModule(fixExportClass_1.default(code, id), {
                fileName: id,
                reportDiagnostics: true,
                compilerOptions: options
            });
            var diagnostics = transformed.diagnostics.filter(goodErrors);
            var fatalError = false;
            diagnostics.forEach(function (diagnostic) {
                var message = typescript.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
                if (diagnostic.file) {
                    var _a = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start), line = _a.line, character = _a.character;
                    console.error(diagnostic.file.fileName + "(" + (line + 1) + "," + (character + 1) + "): error TS" + diagnostic.code + ": " + message);
                }
                else {
                    console.error("Error: " + message);
                }
                if (diagnostic.category === ts.DiagnosticCategory.Error) {
                    fatalError = true;
                }
            });
            if (fatalError) {
                throw new Error("There were TypeScript errors transpiling \"" + id + "\"");
            }
            return {
                code: transformed.outputText +
                    "\nimport { __extends, __decorate, __metadata, __param, __awaiter } from 'typescript-helpers';",
                map: JSON.parse(transformed.sourceMapText)
            };
        }
    };
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = typescript;
